def manage_api_queries(query_string):
    # Manage "classic" API query.
    limit = query_string.get('limit')
    offset = query_string.get('offset')
    sort = [q for q in query_string.get('_sort').split(',')] if query_string.get('_sort') else None
    full_text_search = query_string.get('_q')
    search = [(q[0], '==', q[1]) for q in query_string.items() if q[0] not in ('limit', 'offset', '_sort', '_q')]

    if (limit and not limit.isdigit()) or (offset and not offset.isdigit()):
        raise KeyError

    return limit, offset, search, sort, full_text_search


def manage_datatables_queries(query_string):
    # Manage datatables queries.
    # Draw is a datables variable that needs to be returned in the response.
    draw = int(query_string.get('draw'))
    offset = query_string.get('start')
    limit = query_string.get('length')

    if not draw or not offset or not limit:
        raise KeyError

    full_text_search = query_string.get('search[value]')

    search = []
    search_qs = query_string.get('filter')
    if search_qs:
        if '==' in search_qs:
            params = search_qs.split('==')
            search = [(params[0], '==', params[1])]
        elif '!=' in search_qs:
            params = search_qs.split('!=')
            search = [(params[0], '!=', params[1])]
        else:
            raise KeyError

    # To define the query order, Datatables sends order[sort_order][column] = column_number and
    # order[sort_order][dir] = 'asc' or 'desc'. So we loop through the different keys and format the data according
    # to our needs. (Attribute names are sent in columns[column_number][data]).
    sort = []
    i = 0
    while query_string.get('order[' + str(i) + '][column]'):
        column = str(query_string.get('order[' + str(i) + '][column]'))
        attribute = query_string.get('columns[' + column + '][data]')
        direction = '-' if query_string.get('order[' + str(i) + '][dir]') == 'desc' else ''
        sort.append(direction + attribute)
        i += 1

    return draw, limit, offset, search, sort, full_text_search
