from configparser import ConfigParser
from os import getcwd
from sys import argv

from celery import Celery, signals
from pyramid.settings import aslist


def get_blobstore_path(config):
    for index, arg in enumerate(argv):
        if arg == '-A':
            blob_store_key = argv[index + 1] + '.blobstore_path'
            return config['app:main'][blob_store_key]


def config_file_argument(parser):
    parser.add_argument('--ini', required=True, help='App ini configuration file.')


def configure_celery_app(config_file):
    config = ConfigParser({'here': getcwd()})
    config.read(config_file)

    app.conf.broker_url = config['celery']['broker_url']

    if config['celery'].get('celery_imports'):
        app.conf.imports += aslist(config['celery']['celery_imports'])

    app.conf.sqlalchemy_url = config['app:main']['sqlalchemy.url']

    blobstore_path = get_blobstore_path(config)
    if blobstore_path:
        app.conf.blobstore_path = blobstore_path


app = Celery()
app.user_options['preload'].add(config_file_argument)
app.conf.imports = ['parsys_utilities.celery_tasks']


# noinspection PyUnusedLocal
@signals.user_preload_options.connect
def on_preload_parsed(options, **kwargs):
    configure_celery_app(options['ini'])
