from celery.utils.log import get_task_logger
from requests import ConnectionError, HTTPError, post, Timeout

from parsys_utilities.celery_app import app


logger = get_task_logger(__name__)


@app.task(bind=True)
def post_json(self, server_url, json, username=None, password=None):
    try:
        auth = (username, password) if username and password else None
        post(server_url, json=json, auth=auth)
    except (ConnectionError, HTTPError, Timeout) as exc:
        logger.info('External server ({}) post failed: {}.'.format(server_url, exc))
        self.retry(countdown=30, exc=exc, max_retries=5)


@app.task(bind=True)
def post_mailgun_email(self, server_url, auth_key, **kwargs):
    try:
        post(server_url, auth=('api', auth_key), **kwargs)
    except (ConnectionError, HTTPError, Timeout) as exc:
        logger.info('Mailgun post failed: {}.'.format(exc))
        self.retry(countdown=30, exc=exc, max_retries=5)
