from parsys_utilities.celery_tasks import post_mailgun_email


DEFAULT_CLOUD_NAME = 'Parsys Cloud'


class Email(object):
    def __init__(self, request, cloud_name=DEFAULT_CLOUD_NAME, send_emails=True):
        self._send_emails = send_emails
        self._mailgun_api_key = request.registry.settings['mail.mailgun_api_key']
        self._mailgun_domain = request.registry.settings['mail.mailgun_domain']

        self.default_sender_email = request.registry.settings['mail.default_sender']
        self.default_sender = '{} <{}>'.format(cloud_name, self.default_sender_email)

    def send(self, to, subject, text, html=None, cc=None, bcc=None, sender=None, attachments=()):
        if not self._send_emails:
            return

        if not to or not subject or not text:
            return

        file_attachments = list()

        for attachment in attachments:
            file_attachments.append(('attachment', attachment))

        properties = dict(subject=subject, text=text)
        properties['from'] = sender or self.default_sender

        if isinstance(to, str):
            properties['to'] = to
        elif isinstance(to, list):
            properties['to'] = self.default_sender_email
            properties['bcc'] = to

        if cc:
            properties['cc'] = cc

        if bcc and properties.get('bcc') and isinstance(bcc, str):
            properties['bcc'].append(bcc)
        elif bcc and properties.get('bcc') and isinstance(bcc, list):
            properties['bcc'].extend(bcc)
        elif bcc and not properties.get('bcc'):
            properties['bcc'] = bcc

        if html:
            properties['html'] = html

        post_mailgun_email.apply_async(
            args=['https://api.mailgun.net/v3/{}/messages'.format(self._mailgun_domain), self._mailgun_api_key],
            kwargs={'files': file_attachments, 'data': properties},
            queue='notifications'
        )
