from logging import getLogger, LoggerAdapter


class ContextAdapter(LoggerAdapter):
    def process(self, msg, kwargs):
        if isinstance(msg, list):
            msg = ','.join([str(string) for string in msg])
        return '{ip},{session_id},{user_id},{msg}'.format(**self.extra, msg=msg), kwargs


def logger(request, name=__package__):
    app_logger = getLogger(name)

    user_id = None
    if isinstance(request.user, dict):
        user_id = request.user['id']
    elif isinstance(request.user, tuple):
        user_id = request.user.user_id

    client_addr = request.headers.get('x-real-ip', 'CA+{}'.format(request.client_addr))

    extra = {'ip': client_addr, 'session_id': request.session.get('session_id'), 'user_id': user_id}

    return ContextAdapter(app_logger, extra)


def utilities_logger(request):
    if hasattr(request, 'logger_actions'):
        return request.logger_actions
    else:
        return logger(request)
