import depot.fields.sqlalchemy
# noinspection PyUnresolvedReferences
import sqlalchemy
# noinspection PyUnresolvedReferences
import sqlalchemy.orm
import sqlalchemy.ext.hybrid
from inflection import underscore
from sqlalchemy.ext.declarative import declarative_base, DeclarativeMeta
from sqlalchemy.schema import MetaData

and_ = sqlalchemy.and_
asc = sqlalchemy.asc
backref = sqlalchemy.orm.backref
Boolean = sqlalchemy.Boolean
case = sqlalchemy.case
column_property = sqlalchemy.orm.column_property
Date = sqlalchemy.Date
DateTime = sqlalchemy.DateTime
declared_attr = sqlalchemy.ext.declarative.declared_attr
desc = sqlalchemy.desc
Enum = sqlalchemy.Enum
Field = sqlalchemy.Column
File = depot.fields.sqlalchemy.UploadedFileField
Float = sqlalchemy.Float
ForeignKey = sqlalchemy.ForeignKey
ForeignKeyConstraint = sqlalchemy.ForeignKeyConstraint
func = sqlalchemy.func
hybrid_method = sqlalchemy.ext.hybrid.hybrid_method
hybrid_property = sqlalchemy.ext.hybrid.hybrid_property
Integer = sqlalchemy.Integer
join = sqlalchemy.join
joinedload = sqlalchemy.orm.joinedload
not_ = sqlalchemy.not_
or_ = sqlalchemy.or_
relationship = sqlalchemy.orm.relationship
select = sqlalchemy.select
String = sqlalchemy.Unicode
Table = sqlalchemy.Table
Text = sqlalchemy.Text
Time = sqlalchemy.Time


class _ModelBase(object):
    id = Field(Integer, primary_key=True)


class _DeclarativeMeta(DeclarativeMeta):
    def __new__(mcs, name, bases, d):
        d['__tablename__'] = d.get('__tablename__') or d.get('__table__') or underscore(name)
        return DeclarativeMeta.__new__(mcs, name, bases, d)


# Recommended naming convention used by Alembic, as various different database
# providers will autogenerate vastly different names making migrations more
# difficult. See: http://alembic.readthedocs.org/en/latest/naming.html
NAMING_CONVENTION = {
    "ix": 'ix_%(column_0_label)s',
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    "ck": "ck_%(table_name)s_%(constraint_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s"
}

metadata = MetaData(naming_convention=NAMING_CONVENTION)

Model = declarative_base(cls=_ModelBase, name='ObjectModel', metaclass=_DeclarativeMeta, metadata=metadata)


class AliasMixin(object):
    first_name = Field(String)
    last_name = Field(String)

    @hybrid_property
    def full_name(self):
        if self.last_name and self.first_name:
            return self.last_name + ' ' + self.first_name
        elif self.last_name:
            return self.last_name
        elif self.first_name:
            return self.first_name

    # noinspection PyMethodParameters
    @full_name.expression
    def full_name(cls):
        return case(
            [(cls.last_name.isnot(None) & cls.first_name.isnot(None), cls.last_name + ' ' + cls.first_name),
             (cls.last_name.isnot(None), cls.last_name),
             (cls.first_name.isnot(None), cls.first_name)],
            else_=None)


class CreationDateTimeMixin(object):
    created_at = Field(DateTime, default=func.now(), nullable=False)


class UpdateDateTimeMixin(object):
    updated_at = Field(DateTime, default=func.now(), onupdate=func.now(), nullable=False)
